import { resolve } from 'node:path';
import { useNuxt, logger, defineNuxtModule, createResolver, addComponent, addPlugin } from '@nuxt/kit';
import chalk from 'chalk';
import { execSync } from 'node:child_process';
import { createRequire } from 'node:module';
import consola from 'consola';
import { readPackageJSON } from 'pkg-types';
import { getPackageInfo } from 'local-pkg';
import semver from 'semver';

const version = "0.3.0";

const components = {
  "aspect-ratio": ["OkuAspectRatio"],
  "avatar": ["OkuAvatar", "OkuAvatarFallback", "OkuAvatarImage"],
  "checkbox": ["OkuCheckbox", "OkuCheckboxIndicator"],
  "collapsible": ["OkuCollapsible", "OkuCollapsibleContent", "OkuCollapsibleTrigger"],
  "dialog": ["OkuDialog", "OkuDialogClose", "OkuDialogContent", "OkuDialogDescription", "OkuDialogOverlay", "OkuDialogPortal", "OkuDialogTitle", "OkuDialogTrigger"],
  "hover-card": ["OkuHoverCard", "OkuHoverCardArrow", "OkuHoverCardContent", "OkuHoverCardPortal", "OkuHoverCardTrigger"],
  "label": ["OkuLabel"],
  "popover": ["OkuPopover", "OkuPopoverArrow", "OkuPopoverClose", "OkuPopoverContent", "OkuPopoverPortal", "OkuPopoverTrigger"],
  "progress": ["OkuProgress", "OkuProgressIndicator"],
  "radio-group": ["OkuRadioGroup", "OkuRadioGroupIndicator", "OkuRadioGroupItem"],
  "separator": ["OkuSeparator"],
  "slider": ["OkuSlider", "OkuSliderRange", "OkuSliderThumb", "OkuSliderTrack"],
  "switch": ["OkuSwitch", "OkuSwitchThumb"],
  "tabs": ["OkuTabs", "OkuTabsContent", "OkuTabsList", "OkuTabsTrigger"],
  "toast": ["OkuToast", "OkuToastAction", "OkuToastDescription", "OkuToastProvider", "OkuToastTitle", "OkuToastViewport"],
  "toggle": ["OkuToggle"],
  "toggle-group": ["OkuToggleGroup", "OkuToggleGroupItem"],
  "toolbar": ["OkuToolbar", "OkuToolbarButton", "OkuToolbarLink", "OkuToolbarSeparator", "OkuToolbarToggleGroup"],
  "tooltip": ["OkuTooltip", "OkuTooltipArrow", "OkuTooltipContent", "OkuTooltipPortal", "OkuTooltipProvider", "OkuTooltipTrigger"],
  "accordion": ["OkuAccordion", "OkuAccordionContent", "OkuAccordionHeader", "OkuAccordionImpl", "OkuAccordionImplSingle", "OkuAccordionImplMultiple", "OkuAccordionItem", "OkuAccordionTrigger"],
  "alert-dialog": ["OkuAlertDialog", "OkuAlertDialogAction", "OkuAlertDialogCancel", "OkuAlertDialogContent", "OkuAlertDialogDescription", "OkuAlertDialogDescriptionWarning", "OkuAlertDialogOverlay", "OkuAlertDialogPortal", "OkuAlertDialogTitle", "OkuAlertDialogTrigger"],
  "scroll-area": ["OkuScrollArea", "OkuScrollAreaCorner", "OkuScrollAreaCornerImpl", "OkuScrollAreaScrollbar", "OkuScrollAreaScrollbarAuto", "OkuScrollAreaScrollbarX", "OkuScrollAreaScrollbarY", "OkuScrollAreaScrollbarHover", "OkuScrollAreaScrollbarImpl", "OkuScrollAreaScrollbarScroll", "OkuScrollAreaScrollbarVisible", "OkuScrollAreaThumb", "OkuScrollAreaThumbImpl", "OkuScrollAreaViewport"]
};
async function getMainPackageJSON(nuxt = useNuxt()) {
  return readPackageJSON(nuxt.options.rootDir);
}
async function checkForUpdateOf(name, current, nuxt = useNuxt()) {
  try {
    if (!current) {
      const require = createRequire(nuxt.options.rootDir);
      const info = await getPackageInfo(name, { paths: require.resolve.paths(name) || void 0 });
      if (!info)
        return;
      current = info.packageJson.version;
    }
    const packument = await import('pacote').then((r) => r.default?.packument || r.packument);
    const manifest = await packument(name);
    const latest = manifest["dist-tags"].latest;
    const needsUpdate = latest !== current && semver.lt(current, latest);
    return {
      name,
      current,
      latest,
      needsUpdate
    };
  } catch (e) {
    logger.warn(`Failed to check for update of ${name}:`);
    logger.warn(e);
  }
}
async function installPackage(settings) {
  let isInstall = false;
  if (typeof settings.installComponents !== "boolean") {
    const arrayComponents = Array.from(Object.keys(settings.installComponents));
    const noInstallPackage = [];
    const installedPackage = [];
    for await (const key of arrayComponents) {
      if ((settings.packageJson.dependencies && !Object.prototype.hasOwnProperty.call(settings.packageJson.dependencies, `@oku-ui/${key}`)) ?? (settings.packageJson.devDependencies && !Object.prototype.hasOwnProperty.call(settings.packageJson.devDependencies, `@oku-ui/${key}`)))
        noInstallPackage.push(key);
      else
        installedPackage.push(key);
    }
    if (noInstallPackage.length > 0) {
      execSync(`ni ${noInstallPackage.map((key) => `@oku-ui/${key}`).join(" ")}`, {
        cwd: settings.rootDir,
        stdio: "inherit"
      });
      isInstall = true;
    } else {
      consola.info(`@oku-ui/${installedPackage.join(", @oku-ui/")}${installedPackage.length > 1 ? " are" : " is"} already installed`);
      isInstall = true;
    }
  } else {
    if ((settings.packageJson.dependencies && !Object.prototype.hasOwnProperty.call(settings.packageJson.dependencies, "@oku-ui/primitives")) ?? (settings.packageJson.devDependencies && !Object.prototype.hasOwnProperty.call(settings.packageJson.devDependencies, "@oku-ui/primitives"))) {
      consola.info("@oku-ui/primitives is not installed");
      execSync("ni @oku-ui/primitives", {
        cwd: settings.rootDir,
        stdio: "inherit"
      });
      isInstall = true;
    } else {
      consola.info("@oku-ui/primitives already installed");
      isInstall = true;
    }
  }
  return {
    isInstall
  };
}

const module = defineNuxtModule({
  meta: {
    name: "@oku-ui/primitives-nuxt",
    configKey: "primitives",
    nuxt: "^3.5.0"
  },
  // Default configuration options of the Nuxt module
  defaults: {
    installComponents: {
      checkbox: true
    },
    splash: true,
    autoInstall: true
  },
  async setup(options, nuxt) {
    const resolver = createResolver(import.meta.url);
    const rootDir = resolve(nuxt.options.rootDir);
    let isInstall = false;
    if (options.autoInstall) {
      const data = await getMainPackageJSON(nuxt);
      if (data) {
        const { isInstall: install } = await installPackage({
          packageJson: data,
          rootDir,
          installComponents: options.installComponents
        });
        isInstall = install;
      }
    } else {
      isInstall = true;
    }
    if (isInstall) {
      if (typeof options.installComponents !== "boolean") {
        Object.keys(options.installComponents).forEach((key) => {
          if (Object.prototype.hasOwnProperty.call(options.installComponents, key)) {
            if (components[key]) {
              components[key].forEach((component) => {
                addComponent({
                  name: component,
                  export: component,
                  filePath: `@oku-ui/${key}`
                });
              });
            }
          }
        });
      } else {
        Object.keys(components).forEach((key) => {
          components[key].forEach((component) => {
            addComponent({
              name: component,
              export: component,
              filePath: "@oku-ui/primitives"
            });
          });
        });
      }
    }
    if (options.splash) {
      logger.log("");
      let latestTag = `v${version}`;
      try {
        await checkForUpdateOf("@oku-ui/primitives-nuxt", version, nuxt).then(
          (info) => {
            if (info?.needsUpdate)
              latestTag = `v${info.latest}`;
          }
        );
      } catch (e) {
      }
      logger.log(`${chalk.green("Oku Primitives")} ${chalk.yellow(`v${version}`)} \u2022 Oku ${chalk.gray(`by ${chalk.underline("@productdevbook")}`)}`);
      if (latestTag !== `v${version}`)
        logger.log(`${chalk.gray("  \u251C\u2500 ")}\u{1F389} New version available!${chalk.gray(` Run ${chalk.underline(`pnpm i @oku-ui/primitives-nuxt${latestTag}`)} to update.`)}`);
      logger.log(chalk.dim("  \u2514\u2500 \u{1F496} Like this package? Consider sponsoring me on GitHub https://github.com/sponsors/productdevbook"));
      logger.log("");
    }
    addPlugin(resolver.resolve("./runtime/plugin"));
  }
});

export { module as default };
